﻿Type=Service
Version=2.71
@EndOfDesignText@
#Region  Service Attributes 
	#StartAtBoot: False
#End Region

Sub Process_Globals
	Public MyIP As String = "N/A"
	Public WifiStatus As String = "Disconnected"
	Public BTStatus  As String = "Disconnected"
	Public WifiConnected, BTConnected As Boolean
	Private admin As BluetoothAdmin
	Private serial1 As Serial
	Private socket1 As Socket
	Private server As ServerSocket
	Private port As Int = 21341
	Private uuid As String = "dabcabcd-afac-11de-8a39-0800200c9a6"
	Private astream As AsyncStreams
	Private pe As PhoneEvents
	Private audioStream As AudioStreamer
	Private sendingAudio As Boolean
End Sub

Sub Service_Create
	'start listening for BT and wifi connections
	server.Initialize(port, "server")
	Try
		server.Listen
	Catch
		WifiStatus = "Error listening: " & LastException
		UpdateUI
	End Try
	admin.Initialize("admin")
	serial1.Initialize("serial1")
	If serial1.IsEnabled Then serial1.Listen2("na", uuid)
	pe.Initialize("pe")
	pe_ConnectivityChanged("", "", Null)
	audioStream.Initialize("AudioStream", 22050, True, 16, audioStream.VOLUME_MUSIC)
End Sub

Sub admin_StateChanged (NewState As Int, OldState As Int)
	If NewState = admin.STATE_ON Then serial1.Listen2("na", uuid)
End Sub

Sub pe_ConnectivityChanged (NetworkType As String, State As String, Intent As Intent)
	MyIP = server.GetMyWifiIP
	UpdateUI
End Sub

Sub Service_Start (StartingIntent As Intent)

End Sub
Private Sub UpdateUI
	CallSub(Main, "UpdateUI")
End Sub

Public Sub Disconnect
	If WifiConnected OR BTConnected Then
		astream.Close
		AStream_Terminated
	End If
End Sub
Public Sub ConnectBT(Address As String)
	serial1.Connect2(Address, uuid)
	BTStatus = "Trying to connect..."
	UpdateUI
End Sub

Public Sub ConnectWifi(Ip As String)
	socket1.Initialize("socket1")
	socket1.Connect(Ip, port, 30000)
	WifiStatus = "Trying to connect..."
	UpdateUI
End Sub



Private Sub socket1_Connected (Successful As Boolean)
	'client connected to server
	If Successful Then
		WifiConnected = True
		StartAStream(socket1.InputStream, socket1.OutputStream)
		WifiStatus = "Connected"
	Else
		WifiStatus = "Error: " & LastException
	End If
	UpdateUI
End Sub

Private Sub server_NewConnection (Successful As Boolean, NewSocket As Socket)
	'server accepted client
	If Successful Then
		WifiConnected = True
		StartAStream(NewSocket.InputStream, NewSocket.OutputStream)
		WifiStatus = "Connected"
	Else
		WifiStatus = "Error: " & LastException
	End If
	UpdateUI
	server.Listen
End Sub

Private Sub serial1_Connected (Success As Boolean)
	If Success Then
		BTStatus = "Connected"
		BTConnected = True
		StartAStream(serial1.InputStream, serial1.OutputStream)
	Else
		BTStatus = "Error: " & LastException.Message
	End If
	UpdateUI
End Sub

Private Sub StartAStream (In As InputStream, out As OutputStream)
	Log("StartAStream")
	astream.InitializePrefix(In, True, out, "astream")
	'call StartPlaying so the audio streamer will be ready to receive audio data
	audioStream.StartPlaying
End Sub

Public Sub SendAudio
	audioStream.StartRecording
	sendingAudio = True
End Sub

Public Sub StopSendingAudio
	audioStream.StopRecording
	sendingAudio = False
End Sub


Sub AudioStream_RecordBuffer (Data() As Byte)
	If sendingAudio Then
		astream.Write(Data)
	End If
End Sub

Sub astream_NewData (Buffer() As Byte)
	If sendingAudio = False Then
		'play the received audio data
		audioStream.Write(Buffer)
	End If
End Sub


Sub Astream_Error
	Log("Error: " & LastException)
	astream.Close
	AStream_Terminated 'manually call this method as it will not be called
	'when we explicitly close the connection.
End Sub

Sub AStream_Terminated
	If BTConnected Then
		BTStatus = "Disconnected"
	Else If WifiConnected Then
		WifiStatus = "Disconnected"
	End If
	BTConnected = False
	WifiConnected = False
	audioStream.StopPlaying
	audioStream.StopRecording
	UpdateUI
End Sub

Sub Service_Destroy

End Sub

